import java.util.*;

public class Horse {
	private int horseId;
	private String name;
	private Box box; //Asocjacja binarna - liczno 1
	
	private static int counter = 0;
	private static ArrayList<Horse> horses = new ArrayList<Horse>(); //Ekstensja
	
	//Konstruktor
	public Horse(String name, Box box) throws Exception{
		if(box == null || !Box.getBoxes().contains(box)){
			throw new Exception("new Horse(String, Box): box cannot be null");
		}
		counter++;
		setId(counter);
		setName(name);
		setBox(box);
		horses.add(this);
	}

	//Ustawianie i odczytywanie wartoci atrybutw
	public int getId(){
		return this.horseId;
	}
	private void setId(int id){
		this.horseId = id;
	}
	public String getName(){
		return this.name;
	}
	public void setName(String name){
		this.name = name;
	}
	public Box getBox(){
		return this.box;
	}
	public int getBoxId(){
		return this.box.getId();
	}
	public void setBox(Box box){
		this.box = box;
		if(box.getHorse() != this)
			box.setHorse(this);
	}
	
	//Obsuga ekstensji
	public static ArrayList<Horse> getHorses(){
		return horses;
	}
	public static void removeHorse(Horse horse){
		horses.remove(horse);
		//Usu rwnie wszystkie pary powizane z tym koniem
		for(HorseRiderPair pair : horse.getPairs()){
			HorseRiderPair.removePair(pair);
		}
	}
	
	//Asocjacja z atrybutem HorseRiderPair
	public Set<HorseRiderPair> getPairs(){
		return HorseRiderPair.getPairs(this);
	}
	public Set<Rider> getRiders(){
		return HorseRiderPair.getRiders(getPairs());
	}
	public boolean addRider(Rider rider){
		try{
			HorseRiderPair.addPair(this, rider);
			return true;
		} catch(Exception e){
			return false;
		}
	}
	public boolean addRider(Rider rider, Calendar from){
		try{
			HorseRiderPair.addPair(this, rider, from);
			return true;
		} catch(Exception e){
			return false;
		}
	}
	public boolean addRider(Rider rider, Calendar from, Calendar to){
		try{
			HorseRiderPair.addPair(this, rider, from, to);
			return true;
		} catch(Exception e){
			return false;
		}
	}
	public void removeAllCurrentPairs(){
		for(HorseRiderPair pair : HorseRiderPair.getCurrentPairs(this)){
			HorseRiderPair.removePair(pair);
		}
	}
	
	//Przeciona metoda toString
	public String toString(){
		return "Ko o imieniu " + this.getName() + " z boksu nr " + this.getBoxId();
	}
}
