import java.text.SimpleDateFormat;
import java.util.*;

public class HorseRiderPair {
	private Calendar from, to;
	
	private Horse horse; //Asocjacja liczno 1
	private Rider rider; //Asocjacja liczno 1
	
	private static Set<HorseRiderPair> pairs = new HashSet<HorseRiderPair>(); //Ekstensja
	
	//Prywatne konstruktory
	private HorseRiderPair(Horse horse, Rider rider){
		setHorse(horse);
		setRider(rider);
		setFromDate(getCurrentDate());
		setToDate(null);
		pairs.add(this);
	}
	private HorseRiderPair(Horse horse, Rider rider, Calendar from){
		setHorse(horse);
		setRider(rider);
		setFromDate(from);
		setToDate(null);
		pairs.add(this);
	}
	private HorseRiderPair(Horse horse, Rider rider, Calendar from, Calendar to){
		setHorse(horse);
		setRider(rider);
		setFromDate(from);
		setToDate(to);
		pairs.add(this);
	}
	
	//Ustawianie i odczytywanie wartoci atrybutw
	public Horse getHorse(){
		return this.horse;
	}
	public int getHorseId(){
		return this.horse.getId();
	}
	private void setHorse(Horse horse){
		this.horse = horse;
	}
	public Rider getRider(){
		return this.rider;
	}
	public int getRiderId(){
		return this.rider.getId();
	}
	private void setRider(Rider rider){
		this.rider = rider;
	}
	public Calendar getFromDate(){
		return this.from;
	}
	public String getFromDateString(){
		return String.format("%1$te.%1$tm.%1$tY", this.from);
	}
	private void setFromDate(Calendar from){
		this.from = Calendar.getInstance();
		this.from.clear();
		this.from.set(from.get(Calendar.YEAR), from.get(Calendar.MONTH), from.get(Calendar.DAY_OF_MONTH));
	}
	public Calendar getToDate(){
		return this.to;
	}
	public String getToDateString(){
		return String.format("%1$te.%1$tm.%1$tY", this.to);
	}
	public void setToDate(Calendar to){
		if(to == null){
			this.to = null;
			return;
		}
		this.to = Calendar.getInstance();
		this.to.clear();
		this.to.set(to.get(Calendar.YEAR), to.get(Calendar.MONTH), to.get(Calendar.DAY_OF_MONTH));
	}

	//Obsuga ekstensji, tworzenie nowych par i usuwanie istniejcych
	public static Set<HorseRiderPair> getPairs(){
		return pairs;
	}
	public static Set<HorseRiderPair> getPairs(Rider rider){
		Set<HorseRiderPair> all = null;
		for(HorseRiderPair pair : pairs){
			if(pair.getRiderId() == rider.getId()){
				if(all == null)
					all = new HashSet<HorseRiderPair>();
				all.add(pair);
			}
		}
		return all;
	}	
	public static Set<HorseRiderPair> getPairs(Horse horse){
		Set<HorseRiderPair> all = null;
		for(HorseRiderPair pair : pairs){
			if(pair.getHorseId() == horse.getId()){
				if(all == null)
					all = new HashSet<HorseRiderPair>();
				all.add(pair);
			}
		}
		return all;
	}
	public static Set<HorseRiderPair> getPairs(Horse horse, Rider rider){
		Set<HorseRiderPair> horseAndRider = null;
		for(HorseRiderPair pair : pairs)
			if(pair.getHorseId() == horse.getId() && pair.getRiderId() == rider.getId()){
				if(horseAndRider == null)
					horseAndRider = new HashSet<HorseRiderPair>();
				horseAndRider.add(pair);
			}
		return horseAndRider;
	}
	public static Set<HorseRiderPair> getCurrentPairs(Rider rider){
		Set<HorseRiderPair> current = null;
		for(HorseRiderPair pair : pairs){
			if(pair.getRiderId() == rider.getId() && !pair.isPastPair()){
				if(current == null){
					current = new HashSet<HorseRiderPair>();
				}
				current.add(pair);
			}
		}
		return current;
	}
	public static Set<HorseRiderPair> getCurrentPairs(Horse horse){
		Set<HorseRiderPair> current = null;
		for(HorseRiderPair pair : pairs){
			if(pair.getHorseId() == horse.getId() && !pair.isPastPair()){
				if(current == null)
					current = new HashSet<HorseRiderPair>();
				current.add(pair);
			}
		}
		return current;
	}
	
	public static Set<Horse> getHorses(Collection<HorseRiderPair> pairs){
		Set<Horse> horses = new HashSet<Horse>();
		for(HorseRiderPair pair : pairs)
			horses.add(pair.getHorse());
		return horses;
	}
	public static Set<Rider> getRiders(Collection<HorseRiderPair> pairs){
		Set<Rider> riders = new HashSet<Rider>();
		for(HorseRiderPair pair : pairs)
			riders.add(pair.getRider());
		return riders;
	}
	
	public static boolean removePair(HorseRiderPair pair){
		
		return pairs.remove(pair);
	}
	
	//Tworzenie nowej pary - tylko, gdy jeszcze nie istnieje
	public static HorseRiderPair addPair(Horse horse, Rider rider) throws Exception{
		Set<HorseRiderPair> currentPairs = getPairs(horse, rider);
		if(currentPairs == null)
			return new HorseRiderPair(horse, rider);
		for(HorseRiderPair pair : currentPairs){
			if(pair.getFromDate() == getCurrentDate() && pair.getToDate() == null)
				throw new Exception("Identical pair already exists.");
		}
		return new HorseRiderPair(horse, rider);
	}
	public static HorseRiderPair addPair(Horse horse, Rider rider, Calendar from) throws Exception{
		Set<HorseRiderPair> currentPairs = getPairs(horse, rider);
		if(currentPairs == null)
			return new HorseRiderPair(horse, rider, from);
		for(HorseRiderPair pair : currentPairs){
			if(pair.getFromDate() == getCurrentDate() && pair.getToDate() == null)
				throw new Exception("Identical pair already exists.");
		}
		return new HorseRiderPair(horse, rider, from);
	}
	public static HorseRiderPair addPair(Horse horse, Rider rider, Calendar from, Calendar to) throws Exception{
		Set<HorseRiderPair> currentPairs = getPairs(horse, rider);
		if(currentPairs == null)
			return new HorseRiderPair(horse, rider, from, to);
		for(HorseRiderPair pair : currentPairs){
			if(pair.getFromDate() == getCurrentDate() && pair.getToDate() == null)
				throw new Exception("Identical pair already exists.");
		}
		return new HorseRiderPair(horse, rider, from, to);
	}
	//Helper methods
	private static Calendar getCurrentDate(){
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.AM_PM, 0);
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.clear(Calendar.HOUR);
		calendar.clear(Calendar.MINUTE);
		calendar.clear(Calendar.SECOND);
		calendar.clear(Calendar.MILLISECOND);
		return calendar;
	}
	
	private boolean isPastPair(){
		if(this.getToDate() != null && this.getToDate().before(getCurrentDate())){
			return true;
		}
		return false;
	}

	//Przeciona metoda toString
	public String toString(){
		StringBuffer buffer = new StringBuffer();
		buffer.append("Para ko - " + this.getHorse().getName() + " i jedziec - " + this.getRider().getName());
		if(this.getFromDate() != null)
			buffer.append(" od " + this.getFromDateString());
		if(this.getToDate() != null)
			buffer.append(" do " + this.getToDateString());
		return buffer.toString();
	}
}
