import helpers.NotUniqueValueException;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Vector;
import java.util.Calendar;

public class Group {
	private int id;
	private String name; //ograniczenie {unique}
	private Set<Rider> members;
	private Rider leader;
	
	private static int counter = 0;
	private static Map<String, Group> groups = new HashMap<String, Group>();
	
	public Group(String name) throws RuntimeException{
		counter++;
		setId(counter);
		setName(name);
		this.members = new HashSet<Rider>();
		this.leader = null;
		addGroup(this);
	}

	//Gettery i settery atrybutw
	public int getId() {
		return id;
	}
	private void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}
	public void setName(String name) throws NotUniqueValueException{
		if(findGroup(name))
			throw new NotUniqueValueException();
		this.name = name;
	}
	
	//Obsuga asocjacji grupa-czonek grupy
	private boolean findRider(Rider rider){
		return this.members.contains(rider);
	}
	public boolean addRider(Rider rider){
		if(!findRider(rider)){
			this.members.add(rider);
			rider.addToGroup(this);
			return true;
		}
		return false;
	}
	public boolean removeRider(Rider rider){
		if(findRider(rider)){
			this.members.remove(rider);
			rider.removeFromGroup(this);
			return true;
		}
		return false;
	}
	
	//Obsuga asocjacji grupa - lider grupy (ograniczenie {subset})
	public Rider getLeader() {
		return leader;
	}

	public void setLeader(Rider leader) throws Exception {
		if(leader == null)
			removeLeader();
		else if(this.leader == leader)
			return;
		else if(findRider(leader)){
			if(this.leader != null)
				this.leader.removeAsLeader(this);
			this.leader = leader;
			leader.makeGroupLeader(this);
		}
		else{
			throw new Exception("Powizanie nadrzdne nie istnieje - jedziec nie jest czci grupy");
		}
	}
	public void removeLeader(){
		this.leader = null;
	}
	
	//Obsuga ekstensji
	private static void addGroup(Group group){
		groups.put(group.name, group);
	}
	public static void removeGroup(Group group) throws Exception {
		if(findGroup(group))
			groups.remove(group);
		else
			throw new Exception("Ta grupa nie naley do ekstensji");
	}
	public static void removeGroup(String name) throws Exception {
		if(findGroup(name))
			groups.remove(groups.get(name));
		else
			throw new Exception("Ta grupa nie naley do ekstensji");
	}
	public static boolean findGroup(Group group){
		return groups.containsValue(group);
	}
	public static boolean findGroup(String name){
		return groups.containsKey(name);
	}
	public static boolean findGroup(int id){
		for(Group g : groups.values())
			if(g.getId() == id)
				return true;
		return false;
	}
	
	
	//Asocjacja z atrybutem Coach-Group realizowana za pomoc klasy poredniej CoachGroup
	public Vector<CoachGroup> getCoachPairs(){
		return CoachGroup.getCoachGroups(this);
	}
	public Set<Coach> getCoaches(){
		Set<Coach> coaches = new HashSet<Coach>();
		for(CoachGroup cg : getCoachPairs())
			coaches.add(cg.getCoach());
		return coaches;
	}
	public void addCoach(Coach coach){
		new CoachGroup(coach, this);
	}
	public void addCoach(Coach coach, Calendar to){
		new CoachGroup(coach, this, to);
	}

	
}
