import helpers.PartWithoutWholeException;
import java.util.Vector;
import java.util.Set;
import java.util.HashSet;

public class Rider {
	private Person person;
	private Vector<Horse> horsesRidden;
	private int experienceLevel;
	
	private Set<Group> groupsMember; //asocjacja grupa - czonek grupy
	private Set<Group> groupLeader; //ograniczenie {subset} - asocjacja nadrzdna to bycie czonkiem grupy
	
	public Rider(Person person) throws PartWithoutWholeException{
		if(person == null)
			throw new PartWithoutWholeException();
		this.person = person;
		this.experienceLevel = 0;
		this.horsesRidden = new Vector<Horse>();
		this.groupsMember = new HashSet<Group>();
		this.groupLeader = new HashSet<Group>();
	}
	
	public Person getPerson(){
		return this.person;
	}
	
	public int getExperienceLevel() {
		return this.experienceLevel;
	}

	public void setExperienceLevel(int experienceLevel) throws Exception{
		if(experienceLevel < this.experienceLevel)
			throw new Exception("Poziom dowiadczenia nie moe spada.");
		this.experienceLevel = experienceLevel;
	}

	public Vector<Horse> getHorsesRidden(){
		return this.horsesRidden;
	}
	public void addHorse(Horse horse){
		horsesRidden.add(horse);
	}
	public boolean rodeHorse(Horse horse){
		return this.horsesRidden.contains(horse);
	}
	
	//implementacja asocjacji i ograniczenia {subset}
	public Set<Group> getGroups(){
		return this.groupsMember;
	}
	private boolean findGroup(Group group){
		if(group == null)
			return false;
		return this.groupsMember.contains(group);
	}
	public boolean addToGroup(Group group){
		if(!findGroup(group)){
			this.groupsMember.add(group);
			group.addRider(this);
			return true;
		}
		return false;
	}
	public boolean removeFromGroup(Group group) {
		if(findGroup(group)){
			this.groupsMember.remove(group);
			group.removeRider(this);
			return true;
		}
		return false;	
	}
	
	private boolean findGroupLeader(Group group){
		if(group == null)
			return false;
		return this.groupLeader.contains(group);
	}
	public void makeGroupLeader(Group group) throws Exception{
		if(findGroup(group) && !findGroupLeader(group)){
			this.groupLeader.add(group);
			group.setLeader(this);
		}
		else
			throw new Exception("Powizanie nadrzdne nie istnieje - jedziec nie jest czonkiem grupy.");
	}
	public boolean removeAsLeader(Group group){
		if(findGroupLeader(group)){
			this.groupLeader.remove(group);
			group.removeLeader();
			return true;
		}
		return false;
	}
	
	public String toString(){
		return "Jedziec " + this.getPerson().getName()
				+ " " + this.getPerson().getSurname();
	}
}
